==> minutemonitor_0.1-1.tar.gz: minutemonitor-0.1/QmlStore.cc <==
/* MinuteMonitor, log duration and display usage for the N900
 * Copyright (c) 2012 David Fries <David@Fries.net> http://david.fries.net/
 * 
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include "QmlStore.h"
#include <QHash>
#include <QString>
#include <QTime>
#include <sstream>
#include <iostream>
#include <tr1/unordered_map>
#include <vector>
#include <algorithm>
#include <ctype.h>

using namespace std;

static int qml_register()
{
	int major=0, minor=1;
	int r=0;
	const char *uri="MinuteMonitor";
	r|=qmlRegisterType<LabelModel>(uri, major, minor, "LabelModel");
	r|=qmlRegisterType<CallType>(uri, major, minor, "CallType");
	return r;
}
static int reg=qml_register();

#define INSERT_ROLE(type) roles.insert(type, #type)
LabelModel::LabelModel(QObject *parent)
	: QAbstractListModel(parent)
{
	QHash<int, QByteArray> roles;
	INSERT_ROLE(start_time);
	INSERT_ROLE(remote_name);
	INSERT_ROLE(minutes);
	INSERT_ROLE(remote_uid);
	INSERT_ROLE(type);
	INSERT_ROLE(rate);
	setRoleNames(roles);
}
#undef INSERT_ROLE

void LabelModel::Resize(int count)
{
	int rows=Data.size();
	if(count == rows)
		return;
	// specify the first or last index, not count
	if(count > rows)
		// first position to be added, last position
		beginInsertRows(QModelIndex(), rows, count-1);
	else
		beginRemoveRows(QModelIndex(), count, rows-1);
	Data.resize(count);
	if(count > rows)
		endInsertRows();
	else
		endRemoveRows();
}

void LabelModel::Set(int row, const LabelData &data)
{
	Data[row]=data;
	QModelIndex index=createIndex(row, 0);
	dataChanged(index, index);
}

void LabelModel::UpdateLabels(const QString &remote_uid,
	CallType::CallTypeEnum type)
{
	QModelIndex index;
	for(size_t i=0; i<Data.size(); ++i)
	{
		if(Data[i].remote_uid == remote_uid)
		{
			Data[i].Type=type;
			index=createIndex(i, 0);
			dataChanged(index, index);
		}
	}
}

QVariant LabelModel::data(const QModelIndex &index, int role) const
{
	if(index.row() < 0 || index.row() >= (int)Data.size())
		return QVariant();

	const LabelData &item=Data[index.row()];
	switch(role)
	{
	case start_time:
		return item.start_time;
	case remote_name:
		return item.remote_name;
	case minutes:
		return item.minutes;
	case remote_uid:
		return item.remote_uid;
	case type:
		return item.Type;
	case rate:
		return item.rate;
	default:
		return QVariant();
	}
}

QString QmlStore::StatsHeader() const
{
	return QString(DateToString(PeriodStart()).c_str()) +
		" to " + DateToString(PeriodStop()).c_str();
}

QString QmlStore::Stats_1() const
{
	// it's anytime or metered as for calls which reduce the number of
	// minutes available
	return QString("Type\nAnytime\nEvening/Weekend\nFree\nVoIP");
}

QString QmlStore::Stats_2() const
{
	return QString("Minutes\n%1\n%2\n%3\n%4")
		.arg(AnytimeUsed())
		.arg(EveningUsed())
		.arg(FreeUsed())
		.arg(VoIPUsed());
}

QString QmlStore::Stats_3() const
{
	return QString("Remaining\n%1\nUnlimited\nUnlimited\nUnlimited")
		.arg(AnytimeAvailable());
}

QString QmlStore::StatsFooter() const
{
	return QString("%1 Calls").arg(DataSet().size()); 
}

void QmlStore::CallLog()
{
	FillLabelList(DataSet());
}

class name_sort
{
public:
	bool Digits(const std::string &name) const
	{
		for(size_t i=0; i<name.size(); ++i)
			if(!isdigit(name[i]))
				return false;
		return true;
	}
	string Normalize(const std::string &name, int &order) const
	{
		int size=name.size();
		if(size==12 && name[0]=='+' && name[1]=='1')
		{
			string sub=name.substr(2);
			if(Digits(sub))
			{
				order=2;
				return sub;
			}
		}
		else if(size==11 && name[0]=='1')
		{
			string sub=name.substr(1);
			if(Digits(sub))
			{
				order=1;
				return sub;
			}
		}
		order=0;
		return name;
	}
	int compare(const string &left, const string &right) const
	{
		int lorder, rorder;
		string lbase=Normalize(left, lorder),
			rbase=Normalize(right, rorder);
		if(lbase < rbase)
			return -1;
		if(lbase == rbase)
			return lorder - rorder;
		return 1;
	}
	// left < right
	// if both sides have a name sort on that, otherwise sort on the uid
	// sort on the 10 digit number without any +1 or 1 prefix, but if
	// the number matches sort <num>, 1<num>, +1<num>
	bool operator()(const DBMonitor::LogEntry *left,
		const DBMonitor::LogEntry *right) const
	{
		int result=compare(left->remote_name, right->remote_name);
		if(result<0)
			return true;
		if(result==0)
		{
			result = compare(left->remote_uid, right->remote_uid);
			if(result < 0 || result==0 && left->rate < right->rate)
				return true;
			return false;
		}
		return false;
	}
	// left < right
	bool operator()(const DBMonitor::LogEntry &left,
		const DBMonitor::LogEntry &right) const
	{
		return operator()(&left, &right);
	}
};

void QmlStore::ContactLog()
{
	// remote_uid to consolidate the entries
	// I would use a map to leave it sorted, but I want it sorted
	// by remote_name not remote_uid, so hash it is.
	typedef std::tr1::unordered_map<std::string, DBMonitor::LogEntry>
		HashType;
	HashType::iterator iter;
	HashType hash;
	std::string key;
	const DBMonitor::LogType &log=DataSet();
	int duration;
	for(size_t i=0; i<log.size(); ++i)
	{
		const DBMonitor::LogEntry &entry=log[i];
		// don't combine different rates even for the same contact
		key=entry.remote_uid + "\0" + (char)('0'+entry.rate);
		iter=hash.find(key);
		if(iter==hash.end())
		{
			hash.insert(make_pair(key, entry));
			continue;
		}
		iter->second.ceil_min += entry.ceil_min;
		iter->second.rate = (DBMonitor::CallRateEnum)(
			iter->second.rate | entry.rate);
		duration=entry.end_time - entry.start_time;
		if(duration <= 0)
			continue;
		// end_time will no longer be the end of the call, but rather
		// be used to find the total time
		iter->second.end_time+=duration;
	}

	vector<DBMonitor::LogEntry*> vec;
	vec.reserve(hash.size());
	for(iter=hash.begin(); iter!=hash.end(); ++iter)
		vec.push_back(&iter->second);
	sort(vec.begin(), vec.end(), name_sort());

	DBMonitor::LogType contacts;
	contacts.reserve(hash.size());
	for(size_t i=0; i<vec.size(); ++i)
		contacts.push_back(*vec[i]);
	FillLabelList(contacts);
}

void QmlStore::LoadTelContacts()
{
	DBMonitor::LogType log=DB.GetAllTelContacts();
	sort(log.begin(), log.end(), name_sort());
	FillLabelList(log);
}

bool QmlStore::LoadPrevious()
{
	bool result=Store::LoadPrevious();
	Updated();
	return result;
}

bool QmlStore::LoadNext()
{
	bool result = Store::LoadNext();
	Updated();
	return result;
}

static const char *s_time_format="h:mm ap";
QString QmlStore::SecToTime(int sec)
{
	QTime t(0, 0, 0);
	// addSecs breaks it out, giving that number of seconds in
	// the constructur doesn't and produces an invalid time.
	t=t.addSecs(sec);
	return t.toString(s_time_format);
}

int QmlStore::TimeToSec(const QString &str)
{
	QTime t=t.fromString(str, s_time_format);
	return (t.hour()*60+t.minute())*60+t.second();
}

void QmlStore::SetEveningEnd(const QString &t)
{
	DB.SetEveningEnd(TimeToSec(t));
}

QString QmlStore::GetEveningEnd() const
{
	return SecToTime(DB.GetEveningEnd());
}

void QmlStore::SetEveningBegin(const QString &t)
{
	DB.SetEveningBegin(TimeToSec(t));
}

QString QmlStore::GetEveningBegin() const
{
	return SecToTime(DB.GetEveningBegin());
}

bool QmlStore::WriteConfig()
{
	int result = DB.WriteConfig();
	// The start/end may have changed, reset the period, though we didn't
	// keep the original time, try half way between avoiding overflowing
	// the data type.
	SetPeriod(PeriodStart() + (PeriodStop() - PeriodStart())/2);
	Load();
	Updated();
	return result;
}

void QmlStore::AddFreeContact(const QString &remote_uid)
{
	DB.AddFreeContact(remote_uid.toStdString());
	Model.UpdateLabels(remote_uid, CallType::CALL_FREE);
}

void QmlStore::RemoveFreeContact(const QString &remote_uid)
{
	DB.RemoveFreeContact(remote_uid.toStdString());
	Model.UpdateLabels(remote_uid, CallType::CALL_COST);
}

void QmlStore::FillLabelList(const DBMonitor::LogType &log)
{
	int duration;
	QString start_time, remote_name, minutes, remote_uid;
	const DBMonitor::ContactsHashType &free_call_set=DB.GetFreeContacts();
	CallType::CallTypeEnum type;
	Model.Resize(log.size());
	for(size_t i=0; i<log.size(); ++i)
	{
		const DBMonitor::LogEntry &entry=log[i];
		if(entry.start_time)
			start_time=TimestampToString(entry.start_time).c_str();
		else
			start_time.clear();
		remote_name=entry.remote_name.c_str();

		if(entry.end_time)
		{
			duration=entry.end_time - entry.start_time;
			if(duration<0)
				duration=0;
			// HH:MM:SS and a rounded up minutes
			minutes=QString("%1 (%2)")
				.arg(MinutesToString(duration).c_str())
				.arg(entry.ceil_min);
		}
		else
		{
			minutes.clear();
		}
		remote_uid=entry.remote_uid.c_str();
		if(entry.VoIP)
			type=CallType::CALL_VOIP;
		else
			type=CallType(free_call_set, entry.remote_uid);
		Model.Set(i, LabelData(start_time, remote_name,
			minutes, remote_uid, type,
			(CallType::CallRateEnum)entry.rate));
	}
}

CallType::CallTypeEnum QmlStore::CallType(
	const DBMonitor::ContactsHashType &free_call_set,
	const std::string &remote_uid)
{
	if(free_call_set.find(remote_uid) != free_call_set.end())
		return CallType::CALL_FREE;
	else
		return CallType::CALL_COST;
}

==> minutemonitor_0.2-2.tar.gz: minutemonitor-0.2/QmlStore.cc <==
/* MinuteMonitor, log duration and display usage for the N900
 * Copyright (c) 2012 David Fries <David@Fries.net> http://david.fries.net/
 * 
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include "QmlStore.h"
#include <QHash>
#include <QString>
#include <QTime>
#include <sstream>
#include <iostream>
#include <tr1/unordered_map>
#include <vector>
#include <algorithm>
#include <ctype.h>

using namespace std;

static int qml_register()
{
	int major=0, minor=1;
	int r=0;
	const char *uri="MinuteMonitor";
	r|=qmlRegisterType<LabelModel>(uri, major, minor, "LabelModel");
	r|=qmlRegisterType<CallType>(uri, major, minor, "CallType");
	return r;
}
static int reg=qml_register();

#define INSERT_ROLE(type) roles.insert(type, #type)
LabelModel::LabelModel(QObject *parent)
	: QAbstractListModel(parent)
{
	QHash<int, QByteArray> roles;
	INSERT_ROLE(start_time);
	INSERT_ROLE(remote_name);
	INSERT_ROLE(minutes);
	INSERT_ROLE(remote_uid);
	INSERT_ROLE(type);
	INSERT_ROLE(rate);
	INSERT_ROLE(direction);
	setRoleNames(roles);
}
#undef INSERT_ROLE

void LabelModel::Resize(int count)
{
	int rows=Data.size();
	if(count == rows)
		return;
	// specify the first or last index, not count
	if(count > rows)
		// first position to be added, last position
		beginInsertRows(QModelIndex(), rows, count-1);
	else
		beginRemoveRows(QModelIndex(), count, rows-1);
	Data.resize(count);
	if(count > rows)
		endInsertRows();
	else
		endRemoveRows();
}

void LabelModel::Set(int row, const LabelData &data)
{
	Data[row]=data;
	QModelIndex index=createIndex(row, 0);
	dataChanged(index, index);
}

void LabelModel::UpdateLabels(const QString &remote_uid,
	CallType::CallTypeEnum type)
{
	QModelIndex index;
	for(size_t i=0; i<Data.size(); ++i)
	{
		if(Data[i].remote_uid == remote_uid)
		{
			Data[i].type=type;
			index=createIndex(i, 0);
			dataChanged(index, index);
		}
	}
}

QVariant LabelModel::data(const QModelIndex &index, int role) const
{
	if(index.row() < 0 || index.row() >= (int)Data.size())
		return QVariant();

	const LabelData &item=Data[index.row()];
	switch(role)
	{
	case start_time:
		return item.start_time;
	case remote_name:
		return item.remote_name;
	case minutes:
		return item.minutes;
	case remote_uid:
		return item.remote_uid;
	case type:
		return item.type;
	case rate:
		return item.rate;
	case direction:
		return item.direction;
	default:
		return QVariant();
	}
}

QString QmlStore::StatsHeader() const
{
	return QString(TimestampToString(PeriodStart()).c_str()) +
		" to " + TimestampToString(PeriodStop()).c_str();
}

QString QmlStore::Stats_1()
{
	// each StatStr* is one column
	StatStr1="Type\n";
	StatStr2="Minutes\n";
	StatStr3="Remaining\n";

	// it's anytime or metered as for calls which reduce the number of
	// minutes available
	if(DB.GetIncomingFree())
		StatStr1+="Outgoing/Cost\n";
	else
		StatStr1+="Anytime\n";
	StatStr2+=QString("%1\n").arg(AnytimeUsed());
	StatStr3+=QString("%1\n").arg(AnytimeAvailable());

	if(DB.GetNonprimeFree())
	{
		StatStr1+=DB.GetIncomingFree() ? "Out/Nonprime\n" :
			"Evening/Weekend\n";
		StatStr2+=QString("%1\n").arg(EveningUsed());
		StatStr3+="Unlimited\n";
	}

	if(DB.GetIncomingFree())
	{
		StatStr1+="Incoming(Free)\n";
		StatStr2+=QString("%1\n").arg(IncomingUsed());
		StatStr3+="Unlimited\n";
	}

	StatStr1+="Free\n";
	StatStr2+=QString("%1\n").arg(FreeUsed());
	StatStr3+="Unlimited\n";

	// No \n for the last entry or it causes extra blank space.
	StatStr1+="VoIP";
	StatStr2+=QString("%1").arg(VoIPUsed());
	StatStr3+="Unlimited";

	return StatStr1;
}

QString QmlStore::StatsFooter() const
{
	int cell = AnytimeUsed() + IncomingUsed() + EveningUsed() + FreeUsed();
	return QString("Total: Calls %1, Minutes %2 (Cell %3)")
		.arg(DataSet().size())
		.arg(cell + VoIPUsed())
		.arg(cell);
}

void QmlStore::CallLog()
{
	FillLabelList(DataSet());
}

class name_sort
{
public:
	bool Digits(const std::string &name) const
	{
		for(size_t i=0; i<name.size(); ++i)
			if(!isdigit(name[i]))
				return false;
		return true;
	}
	string Normalize(const std::string &name, int &order) const
	{
		int size=name.size();
		if(size==12 && name[0]=='+' && name[1]=='1')
		{
			string sub=name.substr(2);
			if(Digits(sub))
			{
				order=2;
				return sub;
			}
		}
		else if(size==11 && name[0]=='1')
		{
			string sub=name.substr(1);
			if(Digits(sub))
			{
				order=1;
				return sub;
			}
		}
		order=0;
		return name;
	}
	int compare(const string &left, const string &right) const
	{
		int lorder, rorder;
		string lbase=Normalize(left, lorder),
			rbase=Normalize(right, rorder);
		if(lbase < rbase)
			return -1;
		if(lbase == rbase)
			return lorder - rorder;
		return 1;
	}
	// left < right
	// if both sides have a name sort on that, otherwise sort on the uid
	// sort on the 10 digit number without any +1 or 1 prefix, but if
	// the number matches sort <num>, 1<num>, +1<num>
	bool operator()(const DBMonitor::LogEntry *left,
		const DBMonitor::LogEntry *right) const
	{
		int result=compare(left->remote_name, right->remote_name);
		if(result<0)
			return true;
		if(result==0)
		{
			result = compare(left->remote_uid, right->remote_uid);
			if(result < 0 ||
				(result==0 && left->rate < right->rate))
				return true;
			return false;
		}
		return false;
	}
	// left < right
	bool operator()(const DBMonitor::LogEntry &left,
		const DBMonitor::LogEntry &right) const
	{
		return operator()(&left, &right);
	}
};

void QmlStore::ContactLog()
{
	// remote_uid to consolidate the entries
	// I would use a map to leave it sorted, but I want it sorted
	// by remote_name not remote_uid, so hash it is.
	typedef std::tr1::unordered_map<std::string, DBMonitor::LogEntry>
		HashType;
	HashType::iterator iter;
	HashType hash;
	std::string key;
	const DBMonitor::LogType &log=DataSet();
	int duration;
	for(size_t i=0; i<log.size(); ++i)
	{
		const DBMonitor::LogEntry &entry=log[i];
		// don't combine different rates even for the same contact
		key=entry.remote_uid + "\0" + (char)('0'+entry.rate);
		iter=hash.find(key);
		if(iter==hash.end())
		{
			hash.insert(make_pair(key, entry));
			continue;
		}
		iter->second.ceil_min += entry.ceil_min;
		iter->second.rate = (DBMonitor::CallRateEnum)(
			iter->second.rate | entry.rate);
		iter->second.direction = (DBMonitor::CallDirection)(
			iter->second.direction | entry.direction);
		duration=entry.end_time - entry.start_time;
		if(duration > 0)
		{
			// end_time will no longer be the end of the call, but
			// rather be used to find the total time
			iter->second.end_time+=duration;
		}

	}

	vector<DBMonitor::LogEntry*> vec;
	vec.reserve(hash.size());
	for(iter=hash.begin(); iter!=hash.end(); ++iter)
		vec.push_back(&iter->second);
	sort(vec.begin(), vec.end(), name_sort());

	DBMonitor::LogType contacts;
	contacts.reserve(hash.size());
	for(size_t i=0; i<vec.size(); ++i)
		contacts.push_back(*vec[i]);
	FillLabelList(contacts);
}

void QmlStore::LoadTelContacts()
{
	DBMonitor::LogType log=DB.GetAllTelContacts();
	sort(log.begin(), log.end(), name_sort());
	FillLabelList(log);
}

bool QmlStore::LoadPrevious()
{
	bool result=Store::LoadPrevious();
	Updated();
	return result;
}

bool QmlStore::LoadNext()
{
	bool result = Store::LoadNext();
	Updated();
	return result;
}

static const char *s_time_format="h:mm ap";
QString QmlStore::SecToTime(int sec)
{
	QTime t(0, 0, 0);
	// addSecs breaks it out, giving that number of seconds in
	// the constructur doesn't and produces an invalid time.
	t=t.addSecs(sec);
	return t.toString(s_time_format);
}

int QmlStore::TimeToSec(const QString &str)
{
	QTime t=t.fromString(str, s_time_format);
	return (t.hour()*60+t.minute())*60+t.second();
}

void QmlStore::SetEveningEnd(const QString &t)
{
	DB.SetEveningEnd(TimeToSec(t));
}

QString QmlStore::GetEveningEnd() const
{
	return SecToTime(DB.GetEveningEnd());
}

void QmlStore::SetEveningBegin(const QString &t)
{
	DB.SetEveningBegin(TimeToSec(t));
}

QString QmlStore::GetEveningBegin() const
{
	return SecToTime(DB.GetEveningBegin());
}

bool QmlStore::WriteConfig()
{
	int result = DB.WriteConfig();
	// The start/end may have changed, reset the period, though we didn't
	// keep the original time, try half way between avoiding overflowing
	// the data type.
	SetPeriod(PeriodStart() + (PeriodStop() - PeriodStart())/2);
	Load();
	Updated();
	return result;
}

void QmlStore::AddFreeContact(const QString &remote_uid)
{
	DB.AddFreeContact(remote_uid.toStdString());
	Model.UpdateLabels(remote_uid, CallType::CALL_FREE);
}

void QmlStore::RemoveFreeContact(const QString &remote_uid)
{
	DB.RemoveFreeContact(remote_uid.toStdString());
	Model.UpdateLabels(remote_uid, CallType::CALL_COST);
}

void QmlStore::FillLabelList(const DBMonitor::LogType &log)
{
	int duration;
	QString start_time, remote_name, minutes, remote_uid;
	const DBMonitor::ContactsHashType &free_call_set=DB.GetFreeContacts();
	CallType::CallTypeEnum type;
	Model.Resize(log.size());
	for(size_t i=0; i<log.size(); ++i)
	{
		const DBMonitor::LogEntry &entry=log[i];
		if(entry.start_time)
			start_time=TimestampToString(entry.start_time).c_str();
		else
			start_time.clear();
		remote_name=entry.remote_name.c_str();

		if(entry.end_time)
		{
			duration=entry.end_time - entry.start_time;
			if(duration<0)
				duration=0;
			// HH:MM:SS and a rounded up minutes
			minutes=QString("%1 (%2)")
				.arg(MinutesToString(duration).c_str())
				.arg(entry.ceil_min);
		}
		else
		{
			minutes.clear();
		}
		remote_uid=entry.remote_uid.c_str();
		if(entry.VoIP)
			type=CallType::CALL_VOIP;
		else
			type=GetCallType(free_call_set, entry.remote_uid);
		Model.Set(i, LabelData(start_time, remote_name,
			minutes, remote_uid, type,
			(CallType::CallRateEnum)entry.rate,
			(CallType::CallDirection)entry.direction));
	}
}

CallType::CallTypeEnum QmlStore::GetCallType(
	const DBMonitor::ContactsHashType &free_call_set,
	const std::string &remote_uid)
{
	if(free_call_set.find(remote_uid) != free_call_set.end())
		return CallType::CALL_FREE;
	else
		return CallType::CALL_COST;
}
